#                                                         -*- Perl -*-
# Copyright (c) 2017  Kazuhiro Ito
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#

use strict;
use warnings;
use Getopt::Long qw(:config posix_default no_ignore_case);

use English;
use FileHandle;
use DirHandle;
use DBI;
use JSON;

my $force;
GetOptions (
    "force|f" => \$force
    );

my $src_dirname = "dic";
my $out_filename = "lifedic.sqlite";

if (@ARGV > 0) {
    $src_dirname = $ARGV[0];
    if (@ARGV > 1) {
	$out_filename = $ARGV[1];
    }
}

if (-e $out_filename) {
    print "Output file ($out_filename) already exits.\n";
    if ($force) {
	unlink ($out_filename);
    } else {
	print "Exitting.\n";
	exit (-1);
    }
}

my $db_handle = DBI->connect("dbi:SQLite:dbname=$out_filename");

# Create tables
$db_handle->do('CREATE TABLE "01English" (
	       "en_code" VARCHAR(8) PRIMARY KEY  NOT NULL,
	       "en_notation" VARCHAR(101) NOT NULL,
	       "importance_mark" VARCHAR(5),
	       "auto_ref_code" VARCHAR(8),
	       "auto_notation" TEXT,
	       "is_MeSH" VARCHAR(1),
	       "avail_search" VARCHAR(1),
	       "no_etojvoc" VARCHAR(1),
	       "parent_concept_id" VARCHAR(8),
	       "en_key" VARCHAR(101) )');
$db_handle->do('CREATE TABLE "03Eng_Jap" (
	       "en_code" VARCHAR(8) NOT NULL,
	       "en_ja_comennt" VARCHAR(77),
	       "ending" VARCHAR(5),
	       "ja_code" VARCHAR(8) NOT NULL )');
$db_handle->do('CREATE TABLE "04Eng_Thesaurus" (
	       "en_code" VARCHAR(8) NOT NULL,
	       "rel_en_code" VARCHAR(8) NOT NULL )');
$db_handle->do('CREATE TABLE "06Eng_Sentence" (
	       "en_code" VARCHAR(8) NOT NULL DEFAULT (null) ,
	       "priority_order" INTEGER,
	       "usage_notation" TEXT,
	       "usage_trans" VARCHAR(34),
	       "example" TEXT,
	       "example_translation" TEXT)');
$db_handle->do('CREATE TABLE "07Japanese" (
	       "ja_code" VARCHAR(8) PRIMARY KEY  NOT NULL,
	       "ja_notation" VARCHAR(66) NOT NULL,
	       "importance_mark" VARCHAR(5),
	       "standard_disease_flg" VARCHAR(1),
	       "avail_search" VARCHAR(1),
	       "auto_ref_id" VARCHAR(8),
	       "auto_notation" TEXT,
	       "manual_ref_comennt" TEXT,
	       "parent_concept_id" VARCHAR(8) )');
$db_handle->do('CREATE TABLE "08Jap_Yomi" (
	       "ja_code" VARCHAR(8) NOT NULL,
	       "read" TEXT NOT NULL )');
$db_handle->do('CREATE TABLE "10Jap_Thesaurus" (
	       "ja_code" VARCHAR(8) NOT NULL,
	       "rel_ja_code" VARCHAR(8) NOT NULL )');
$db_handle->do('CREATE TABLE "11Eng_Refer" (
	       "en_code" VARCHAR(8) NOT NULL,
	       "manu_ref_en_code" VARCHAR(8) )');

# Create indices
$db_handle->do('CREATE INDEX "03Eng_Jap_idx_en_code" ON
	       "03Eng_Jap" ("en_code" ASC)');
$db_handle->do('CREATE INDEX "04Eng_Thesaurus_idx_en_code" ON
	       "04Eng_Thesaurus" ("en_code" ASC)');
$db_handle->do('CREATE INDEX "06Eng_Sentence_idx_en_codr" ON
	       "06Eng_Sentence" ("en_code" ASC)');
$db_handle->do('CREATE INDEX "08Jap_Yomi_idx_ja_code" ON
	       "08Jap_Yomi" ("ja_code" ASC)');
$db_handle->do('CREATE INDEX "10Jap_Thesaurus_idx_ja_code" ON
	       "10Jap_Thesaurus" ("ja_code" ASC)');
$db_handle->do('CREATE INDEX "11Eng_Refer_idx_en_code" ON
	       "11Eng_Refer" ("en_code" ASC)');

my $src_dir = DirHandle->new($src_dirname);

while (my $in_filename = $src_dir->read()) {
    if ($in_filename =~ /^dic[EJ].*\.js$/) {
	print $in_filename . "\n";
	my $infile = FileHandle->new($src_dirname . "/" . $in_filename);
	my $content = join '', $infile->getlines();
	$content =~ s/\Avar\s*[a-zA-Z_0-9]+\s*=\s*//;
	$content =~ s/\}[^}]+\Z/}/;
	# workaround for garbage
	$content =~ s/\x0b//;
	my $parsed_content = JSON::decode_json($content);
	my $prequery = "";
	my $query = "";
	my $count = 0;

	if ($in_filename =~ /^dicEn(_[0-9]+)?\.js$/) {
	    $prequery = 'INSERT INTO "01English" ' .
		'("en_code","en_notation","importance_mark","en_key") ' .
		"VALUES\n";
	    for my $key (keys %$parsed_content) {
		my %hash = %{%$parsed_content{$key}};
		if (defined($hash{"en_notation"})) {
		    $query .= sprintf("(%s,%s,%s,%s),\n",
				      $db_handle->quote($key),
				      $db_handle->quote($hash{"en_notation"}),
				      maybe_quote(\$db_handle,
						  $hash{"importance_mark"}),
				      maybe_quote(\$db_handle,
						  $hash{"en_key"}));
		    $count += 1;
		}
	    }
	} elsif ($in_filename =~ /^dicEn2Jp(_[0-9]+)?\.js$/) {
	    $prequery = 'INSERT INTO "03Eng_Jap" ' .
		'("en_code","en_ja_comennt","ending","ja_code") VALUES' . "\n";
	    for my $key (keys %$parsed_content) {
		foreach my $href (@{%$parsed_content{$key}}) {
		    my %hash = %$href;
		    if (defined($hash{"ja_code"})) {
			$query .= sprintf("(%s,%s,%s,%s),\n",
					  $db_handle->quote($key),
					  maybe_quote(\$db_handle,
						      $hash{"en_ja_comennt"}),
					  maybe_quote(\$db_handle,
						      $hash{"ending"}),
					  $db_handle->quote($hash{"ja_code"}));
			$count += 1;
		    }
		}
	    }
	} elsif ($in_filename =~ /^dicEnSentence(_[0-9]+)?\.js$/) {
	    $prequery = 'INSERT INTO "06Eng_Sentence" ' .
		'("en_code","priority_order","usage_notation",' .
		  '"usage_trans","example", "example_translation") ' .
		"VALUES\n";
	    for my $key (keys %$parsed_content) {
		foreach my $href (@{%$parsed_content{$key}}) {
		    my %hash = %$href;
		    $query .=
			sprintf("(%s,%s,%s,%s,%s,%s),\n",
				$db_handle->quote($key),
				maybe_quote(\$db_handle,
					    $hash{"priority_order"}),
				maybe_quote(\$db_handle,
					    $hash{"usage_notation"}),
				maybe_quote(\$db_handle, $hash{"usage_trans"}),
				maybe_quote(\$db_handle, $hash{"example"}),
				maybe_quote(\$db_handle,
					    $hash{"example_translation"}));
		    $count += 1;
		}
	    }
	} elsif ($in_filename =~ /^dicEnThesaurus(_[0-9]+)?\.js$/) {
	    $prequery = 'INSERT INTO "04Eng_Thesaurus" ' .
		'("en_code","rel_en_code") VALUES' . "\n";
	    for my $key (keys %$parsed_content) {
		foreach my $rel_code (@{%$parsed_content{$key}}) {
		    $query .= sprintf("(%s,%s),\n",
				      $db_handle->quote($key),
				      $db_handle->quote($rel_code));
		    $count += 1;
		}
	    }
	} elsif ($in_filename =~ /^dicJp(_[0-9]+)?\.js$/) {
	    $prequery = 'INSERT INTO "07Japanese" ' .
		'("ja_code","ja_notation") VALUES' . "\n";
	    for my $key (keys %$parsed_content) {
		$query .= sprintf("(%s,%s),\n",
				  $db_handle->quote($key), 
				  $db_handle->quote($$parsed_content{$key}));
		$count += 1;
	    }
	} elsif ($in_filename =~ /^dicJpYomi(_[0-9]+)?\.js$/) {
	    $prequery = 'INSERT INTO "08Jap_Yomi" ' .
		'("ja_code","read") VALUES' . "\n";
	    for my $key (keys %$parsed_content) {
		$query .= sprintf("(%s,%s),\n",
				  $db_handle->quote($key), 
				  $db_handle->quote($$parsed_content{$key}));
		$count += 1;
	    }
	} elsif ($in_filename =~ /^dicEnRefer(_[0-9]+)?\.js$/) {
	    $prequery = 'INSERT INTO "11Eng_Refer" ' .
		'("en_code","manu_ref_en_code") VALUES' . "\n";
	    for my $key (keys %$parsed_content) {
		foreach my $ref_code (@{%$parsed_content{$key}}) {
		    $query .= sprintf("(%s,%s),\n",
				      $db_handle->quote($key), 
				      $db_handle->quote($ref_code));
		    $count += 1;
		}
	    }
	}
	if (length($query) > 0) {
	    $query =~ s/,\n\Z/;\n/;
	    $db_handle->do($prequery . $query);
	    printf("Register %d entries.\n", $count);
	}
    }
}

sub maybe_quote {
    my ($db_handle, $value) = @_;
    (length($value) == 0) ? "NULL" : $$db_handle->quote($value);
}

